Module core

Module core 

Core traits and types for interface with the queues in this crate.

This module contains functionality which interacts with the underlying implementations of queues. For most use cases it should not be neccessary to use any of this functionality.

Modules

slots
Module containing types used to determine the underlying storage type in nblf-queue Queues. In most cases the Auto type, which is used as default across this crate, should suffice.

Structs

TruncatedU64
An U64, with the upper N bits set to 0.

Traits

AsPackedValue
This trait is used to store the value in a Slot. The slot may truncate the value to MIN_BIT_WIDTH bits. Types implementing AsPackedValue may be stored in slots with MAX_CARGO_BIT_WIDTH >= MIN_BIT_WIDTH. This will be checked at compile time. MIN_BIT_WIDTH cannot be larger than 64