Module containing types used to determine the underlying storage type in nblf-queue Queues.
In most cases the Auto type, which is used as default across this crate, should suffice.
Structs
- Auto
- Slot type which chooses a concrete implementation based on arch and feature flags.
- Tagged64
- Slot type describing a tagged 64 bit value.
Only available if
target_has_atomic = "64"is true or on featureatomic-fallback. - Tagged128
- Slot type describing a tagged 128 bit value.
Only available if
target_has_atomic = "128"is true or on featureatomic-fallback.